<?php
namespace LMS\Tools\Homework\Model;

use LMS\Libraries\Base;
use LMS\Libraries\ServiceLocator;
use LMS\Tools\Homework\Repositories;

defined('_JEXEC') or die;


class Homework extends Base\Model
{
	protected $name = 'Homework';

	public function __construct($options)
	{
		parent::__construct($options);
		$this->modelDir = __DIR__;
		$this->repository = isset($options['repository']) ? $options['repository'] : new Repositories\Homework(array('db' => ServiceLocator::get('db')));
	}

	public function getItem()
	{
		$id = $this->getState('id');
		$item = $this->repository->getByID($id);

		$period_parts = $this->periodParts($item->show_period);
		$item->days = $period_parts['days'];
		$item->hours = $period_parts['hours'];
		$item->minutes = $period_parts['minutes'];

		if(isset($item->homeworkResults) && is_array($item->homeworkResults) && count($item->homeworkResults) && $this->getState('userId')){
			$results = $item->getHomeworkResults(
				array(
					'user_id' => $this->getState('userId')
				)
			);
			if(isset($results[0])){
				$item->result = $results[0];
			}
		}

		return $item;
	}

	public function getForm($data = array(), $loadData = true)
	{
		$form = $this->loadForm('com_joomla_lms.homework', 'homework', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}
		return $form;
	}

	public function loadFormData()
	{
		$item = $this->getItem();

//		$item->hw_id = $item->id;
//		if($item->homework_results->id){
//			$item->id = $item->homework_results->id;
//			$item->write_text = $item->homework_results->write_text;
//		}

		return $item;
	}

	public function save($data=array())
	{
		if(isset($data['id']) && $data['id'] && $this->repository->getByID($data['id'])){
			return $this->repository->update($data, array('id' => $data['id']));
		} else {
			$homeworkList = $this->repository->getByFilter(
				array('courseId'=>$this->getState('courseId'))
			);
			$data['ordering'] = count($homeworkList) + 1;
			if($this->repository->save($data)){
				return $this->repository->getId();
			}
		}
	}

	public function delete($cid)
	{
		$conditions = array();
		$conditions['id'] = $cid;

//		$repositoryHomeworkResults = new Repositories\HomeworkResults();

		$this->repository->delete($conditions);
	}

	protected function periodInMinutes( $day, $hour, $min )
	{
		$res = $day * 24 * 60 + $hour * 60 + $min;
		return $res;
	}

	protected function periodParts( $showperiod )
	{
		$res = array();

		$ost1 = $showperiod % (24 * 60);
		$res['days'] = ($showperiod - $ost1) / (24 * 60);

		$ost2 = $showperiod % 60;
		$res['hours'] = ($ost1 - $ost2) / 60;

		$res['minutes'] = $ost2;

		return $res;
	}

	public function getScript()
	{
		return 'components/com_joomla_lms/tools/homework/models/forms/homework.js';
	}
}