<?php
namespace LMS\Tools\Homework\Model;

use LMS\Libraries\Base;
use LMS\Libraries\ServiceLocator;
use LMS\Tools\Homework\Repositories;

defined('_JEXEC') or die;


class Result extends Base\Model
{
	protected $name = 'Result';
	protected $repository;

	public function __construct($options)
	{
		parent::__construct($options);
		$this->modelDir = __DIR__;
		$this->repository = isset($options['repository']) ? $options['repository'] : new Repositories\HomeworkResults(array('db' => ServiceLocator::get('db')));
	}

	public function getItem()
	{
		if($this->getState('permissions.homework.manage')){
			$id = $this->getState('id');
			$item = $this->repository->getByID($id);
		} else
		if($this->getState('permissions.homework.view')){
			$courseId = $this->getState('courseId');
			$homeworkId = $this->getState('homeworkId');
			$userId = $this->getState('userId');

			$item = $this->repository->getByFilter(
				array(
					'courseId' => $courseId,
					'userId' => $userId,
					'homeworkId' => $homeworkId,
					'resultType' => 'object'
				)
			);
		}

		return $item;
	}

	public function getForm($data = array(), $loadData = true)
	{
		$form = $this->loadForm('com_joomla_lms.homework', 'result', array('control' => 'jform', 'load_data' => $loadData));

		if (empty($form))
		{
			return false;
		}
		return $form;
	}

	public function loadFormData()
	{
		$item = $this->getItem();

		return $item;
	}

	public function mark($data=array())
	{
		$task = $this->getState('task');
		$course_id = $this->getState('course_id');

		$data['hw_status'] = 1;
		$data['hw_date'] = gmdate("Y-m-d H:i:s");

		if($task == 'upload'){
			$file_id = \JLMS_uploadFile($course_id, 'file');
			if(isset($data['id']) && $data['id'] && $this->repository->getByID($data['id'])){
				$result = $this->repository->getByID($data['id']);
			}
			if ($file_id) {
				if ($result->file_id) {
					\JLMS_deleteFiles($result->file_id);
				}
				$data['file_id'] = $file_id;
			}
			$data['hw_status'] = 0;
		}

//		echo '<pre>';
//		print_r($data);
//		echo '</pre>';
//		die;


		if(isset($data['id']) && $data['id'] && $this->repository->getByID($data['id'])){
			return $this->repository->update($data, array('id' => $data['id']));
		} else {
			if($this->repository->save($data)){
				return $this->repository->getId();
			}
		}
	}

	public function check($data=array())
	{
		$data['hw_status'] = 3;
		if($data['grade'] == 'incomplete'){
			$data['hw_status'] = 0;
//			$data['grade'] = \JText::_('Incomplete');
		}

		if(isset($data['id']) && $data['id'] && $this->repository->getByID($data['id'])){
			return $this->repository->update($data, array('id' => $data['id']));
		} else {
			if($this->repository->save($data)){
				return $this->repository->getId();
			}
		}
	}

	public function delete($cid)
	{
		$conditions = array();
		$conditions['id'] = $cid;

		$this->repository->delete($conditions);
	}

	public function seen()
	{
		$id = $this->getState('result.id');
		$status = $this->getState('result.status');

		$data = array(
			'hw_status' => $status
		);

		return $this->repository->update($data, array('id' => $id));
	}
}